from scoutsdk import scout, AbstractDocumentChunker, DocumentChunks, Chunk, ChunkMetadata


@scout.document_chunker(
    priority=2
)  # Lower priority means that this chunker will be validated first
class DemoChunker(AbstractDocumentChunker):
    def supports_document(
        self, url: str
    ) -> bool:  # Return true when your document chunker
        return False

    def process_document(self, url: str) -> DocumentChunks: # Return the chunks to embed the documents        
        return DocumentChunks(chunks=[Chunk(chunk_to_embed="The content transformed into embeddings", 
                                            content_to_return="The content returned to the assistant when the embedding match the request.", metadata=ChunkMetadata(hierarchy=["heading 1", "heading 2"], custom_property="Will be returned to the model"))])



import json
import requests
from typing import Any, Optional, Tuple


class RequestUtils:
    @staticmethod
    def post(
        url: str,
        headers: dict,
        json_payload: Optional[dict] = None,
        data: Optional[dict] = None,
        files: Optional[dict] = None,
        stream: bool = False,
    ) -> Tuple[Any, int]:
        if files is not None and json_payload is not None:
            data = {"json_payload": json.dumps(json_payload)}
            json_payload = None

        response = requests.post(
            url=url, headers=headers, files=files, json=json_payload, data=data
        )
        return RequestUtils._process_response(response, stream=stream)

    @staticmethod
    def put(
        url: str,
        headers: dict,
        payload: Optional[dict] = None,
    ) -> Tuple[Any, int]:
        response = requests.put(url=url, headers=headers, json=payload)
        return RequestUtils._process_response(response)

    @staticmethod
    def get(
        url: str,
        headers: dict,
        stream: bool = False,
        params: Optional[dict] = None,
    ) -> Tuple[Any, int]:
        response = requests.get(url=url, headers=headers, params=params)
        return RequestUtils._process_response(response, stream=stream)

    @staticmethod
    def delete(
        url: str,
        headers: dict,
        json_payload: Optional[dict] = None,
    ) -> Tuple[Any, int]:
        response = requests.delete(url=url, headers=headers, json=json_payload)
        return RequestUtils._process_response(response)

    @staticmethod
    def patch(
        url: str,
        headers: dict,
        json_payload: Optional[dict] = None,
    ) -> Tuple[Any, int]:
        response = requests.patch(url=url, headers=headers, json=json_payload)
        return RequestUtils._process_response(response)

    @staticmethod
    def _process_response(
        response: requests.Response,
        stream: bool = False,
    ) -> Tuple[Any, int]:
        try:
            RequestUtils._check_for_error_response(response)
            if stream:
                return (
                    RequestUtils._handle_stream_response(response=response),
                    response.status_code,
                )
            else:
                content_type = response.headers.get("Content-Type", "")
                if "application/json" in content_type:
                    return (response.json(), response.status_code)
                elif "text/" in content_type:
                    return (response.text, response.status_code)
                else:
                    return (response.content, response.status_code)
        except requests.exceptions.HTTPError as e:
            response_content = response.content.decode("utf-8")
            error_message = f"{str(e)}\n{response_content}"
            raise type(e)(error_message) from e
        except Exception as e:
            error_message = f"{str(e)}\n{response}"
            raise type(e)(error_message) from e

    @staticmethod
    def _handle_stream_response(
        response: requests.Response,
    ) -> Any:
        accumulated_current_data = ""
        for chunk in response.iter_content(chunk_size=8192):
            if chunk:
                decoded_chunk = chunk.decode("utf-8")
                if decoded_chunk.strip() == ":" or len(decoded_chunk) == 0:
                    continue

                current_data = accumulated_current_data + decoded_chunk
                accumulated_current_data = ""

                if not current_data.endswith("\n"):
                    accumulated_current_data += current_data
                    continue

                chunks = current_data.strip().split("\n")

                received_stream_chunks = [
                    json.loads(f"{chunk}")
                    for chunk in chunks
                    if chunk and chunk.strip() != ":"
                ]

                # only return the last chunk if the stream is finished
                for received_chunk in received_stream_chunks:
                    if received_chunk.get("finish_reason") == "stop":
                        return received_chunk

        return {}

    @staticmethod
    def _check_for_error_response(response: requests.Response) -> None:
        response.raise_for_status()

        # look for the error in the response
        content_type = response.headers.get("Content-Type", "")
        if "application/json" in content_type:
            try:
                response_json = response.json()
                if isinstance(response_json, dict):
                    # Check for an 'error' key with a truthy value
                    error_content = response_json.get("error")
                    if error_content:
                        raise Exception(f"API error in JSON response: {error_content}")
            except requests.exceptions.JSONDecodeError:
                raise Exception(f"API error in JSON response: {response}")



import argparse
import json
import logging
from .base_module import BaseModule
from ..config import Config, CONFIG_ASSISTANT_ID
from ..error_handler import ValidationError, APIError, logger
from ...api import ScoutAPI
from ...api.types.chat import ChatCompletionMessage
from typing import Optional


class ChatCompletionModule(BaseModule):
    def __init__(self, config: Config) -> None:
        """Initialize the chat completion module."""
        self.assistant_id = config.get(CONFIG_ASSISTANT_ID, None)

    def get_command(self) -> str:
        return "chat-completion"

    def add_parser(self, subparsers: argparse._SubParsersAction) -> None:
        completion_parser = subparsers.add_parser(
            self.get_command(), help="Send a message to the chat completion API"
        )
        completion_parser.add_argument(
            "-u", "--user-message", type=str, required=True, help="The message to send"
        )
        completion_parser.add_argument(
            "-s", "--system-prompt", type=str, help="Optional system prompt to use"
        )
        completion_parser.add_argument(
            "-m", "--model", type=str, help="The model to use for chat completion"
        )
        completion_parser.add_argument(
            "-str", "--stream", type=bool, default=False, help="Stream the response"
        )
        completion_parser.add_argument(
            "-a",
            "--assistant-id",
            type=str,
            help="The assistant ID to use for chat completion",
        )
        completion_parser.add_argument(
            "-r",
            "--raw",
            action="store_true",
            help="Display raw response",
        )

        completion_parser.set_defaults(func=self.execute)

    def execute(self, args: argparse.Namespace) -> None:
        """Execute the chat completion command."""

        # Validate required parameters
        assistant_id = self._get_assistant_id(args)
        if not assistant_id:
            raise ValidationError(
                "Assistant ID is required. Provide it with --assistant-id or set it in your environment."
            )

        messages: list[ChatCompletionMessage] = []
        system_prompt = self._get_system_prompt(args)
        if system_prompt:
            messages.append(
                ChatCompletionMessage(
                    role="system",
                    content=system_prompt,
                )
            )
        messages.append(ChatCompletionMessage(role="user", content=args.user_message))

        try:
            response = ScoutAPI().chat_completion(
                messages=messages,
                model=self._get_model(args),
                assistant_id=assistant_id,
                stream=self._get_stream(args),
            )
        except Exception as e:
            raise APIError(f"Failed to get chat completion: {str(e)}")

        # Print the response
        logger.info("\nResponse:")
        if args.raw:
            print(json.dumps(response, indent=2))
        else:
            if "messages" in response and len(response["messages"]) > 0:
                print(response["messages"][0]["content"])
            else:
                logger.warning("Received an empty or invalid response")
                if logger.level <= logging.DEBUG:
                    logger.debug(f"Raw response: {json.dumps(response, indent=2)}")

    def _get_assistant_id(self, args: argparse.Namespace) -> Optional[str]:
        """Get the assistant ID from the arguments."""
        if args.assistant_id:
            return args.assistant_id
        return self.assistant_id

    def _get_model(self, args: argparse.Namespace) -> Optional[str]:
        """Get the model from the arguments."""
        if args.model:
            return args.model
        return None

    def _get_system_prompt(self, args: argparse.Namespace) -> Optional[str]:
        """Get the system prompt from the arguments."""
        if args.system_prompt:
            return args.system_prompt
        return None

    def _get_stream(self, args: argparse.Namespace) -> bool:
        """Get the stream from the arguments."""
        if args.stream:
            return args.stream
        return False



import argparse
import shutil
from pathlib import Path
from importlib.resources import files
from importlib.resources.abc import Traversable
from scoutsdk.cli.modules.base_module import BaseModule
from ..error_handler import ValidationError, logger
from scoutsdk.cli.config import Config


class InitModule(BaseModule):
    def __init__(self, config: Config) -> None:
        self.config = config

    def get_command(self) -> str:
        return "init"

    def add_parser(self, subparsers: argparse._SubParsersAction) -> None:
        project_parser = subparsers.add_parser(
            self.get_command(), help="Creates an empty scout custom function project"
        )

        project_parser.add_argument(
            "-f",
            "--function",
            help="Initialize a new Scout function project",
            action="store_true",
        )

        project_parser.add_argument(
            "-a",
            "--assistant",
            help="Initialize a new Scout assistant project",
            action="store_true",
        )

        project_parser.add_argument(
            "-m",
            "--micro-app",
            help="Initialize a new Scout Micro App project",
            action="store_true",
        )

        project_parser.add_argument(
            "-d",
            "--destination",
            help="Destination folder for the new project",
            required=True,
        )

        project_parser.set_defaults(func=self.execute)

    def execute(self, args: argparse.Namespace) -> None:
        self._init_project(args)

    def _copy_item(self, item: Traversable, dest_path: Path) -> None:
        """Copy a file or directory while ignoring __init__.py and __pycache__ files."""
        if item.name in ["__init__.py", "__pycache__"]:
            return

        if item.is_file():
            shutil.copy2(str(item), dest_path)
        else:
            shutil.copytree(
                str(item),
                dest_path / item.name,
                ignore=shutil.ignore_patterns("__init__.py", "__pycache__", "*.pyc"),
            )

    def _init_project(self, args: argparse.Namespace) -> None:
        # Create the Path object for the destination
        destination = args.destination
        dest_path = Path(destination)

        # Create the directory if it doesn't exist
        if not dest_path.exists():
            dest_path.mkdir(parents=True)
            logger.debug(f"Created directory: {destination}")

        # Check if the directory is empty
        if any(dest_path.iterdir()):
            raise ValidationError(
                f"Destination directory '{destination}' is not empty. "
                "Please choose an empty or non-existent directory."
            )

        data_path = files("scoutsdk.data")

        args_function = args.function
        args_assistant = args.assistant
        args_micro_app = args.micro_app

        if not args_function and not args_assistant and not args_micro_app:
            args_function = True
            args_assistant = True
            args_micro_app = True

        shutil.copy2(str(data_path / ".env.template"), dest_path)

        # Copy files based on the selected options
        if args_function:
            function_path = dest_path / "function"
            function_path.mkdir(exist_ok=True)
            functions_path = data_path / "functions"
            for item in functions_path.iterdir():
                self._copy_item(item, function_path)

        if args_assistant:
            assistant_path = dest_path / "assistant"
            assistant_path.mkdir(exist_ok=True)
            assistants_path = data_path / "assistants"
            for item in assistants_path.iterdir():
                self._copy_item(item, assistant_path)

        if args_micro_app:
            micro_app_path = dest_path / "micro-app"
            micro_app_source_path = data_path / "micro-app"
            micro_app_path.mkdir(exist_ok=True)
            for item in micro_app_source_path.iterdir():
                self._copy_item(item, micro_app_path)

        logger.info(f"Successfully initialized project in {destination}")



import argparse
import json
import os
import subprocess
import shutil
from pathlib import Path
from typing import List, Optional
from pydantic import BaseModel
from .base_module import BaseModule
from .pkg_module import PkgModule
from ..config import Config
from ..error_handler import FileSystemError, ModuleError, logger
from ..types import GenericResponse, EmptyRequest
from scoutsdk.api import ScoutAPI


# Pydantic models for the config file
class ConfigAssistantFile(BaseModel):
    filepath: str
    description: Optional[str] = None


class ConfigPackage(BaseModel):
    package_path: str
    package_file_name: str


class ConfigUI(BaseModel):
    ui_path: str
    ui_package_file_name: str
    ui_build_cmd: Optional[str] = None


class ConfigAssistant(BaseModel):
    name: str
    id: str
    description: str
    instructions_text: Optional[str] = None
    instructions_path: Optional[str] = None
    allowed_functions: Optional[List[str]] = None
    allowed_external_services: Optional[List[str]] = None
    use_system_prompt: bool = False
    prompt_starters_text: Optional[List[str]] = None
    prompt_starters_path: Optional[List[str]] = None
    assistant_files: List[ConfigAssistantFile] = []
    avatar_path: Optional[str] = None
    visibility_type: Optional[str] = "private"
    variables: Optional[dict] = None
    secrets: Optional[dict] = None
    secrets_path: Optional[str] = None
    package_info: Optional[ConfigPackage] = None
    ui_url: Optional[str] = None
    ui_info: Optional[ConfigUI] = None

    model_config = {"extra": "forbid"}


class SyncConfig(BaseModel):
    assistants: List[ConfigAssistant]


class AssistantIds(BaseModel):
    id: str
    assistant_id: str


class DeployedUrlIds(BaseModel):
    url: str
    ids: list[AssistantIds]


class DeployedIds(BaseModel):
    ids_by_url: list[DeployedUrlIds]


DEV_IDS_URLS = ["localhost", "127.0.0.1"]
DEV_ASSISTANT_IDS_FILENAME = ".dev.assistant_ids.json"
ASSISTANT_IDS_FILENAME = "assistant_ids.json"


class SyncModule(BaseModule):
    def __init__(self, config: Config, pkg_module: PkgModule):
        self.config = config
        self._pkg_module = pkg_module
        self._deployed_ids_file: Path = Path(ASSISTANT_IDS_FILENAME)
        self._deployed_ids: DeployedIds = DeployedIds(ids_by_url=[])
        self._api_url = self._get_api_url()

    def _get_api_url(self) -> str:
        try:
            scout_context_str = os.environ.get("SCOUT_CONTEXT")
            if scout_context_str:
                scout_context = json.loads(scout_context_str)
                api_url = scout_context.get("SCOUT_API_URL", "")
                if api_url:
                    return api_url
        except Exception as e:
            logger.warning(f"Error parsing SCOUT_CONTEXT: {str(e)}")
        return ""

    def get_command(self) -> str:
        return "synchronize-assistants"

    def add_parser(self, subparsers: argparse._SubParsersAction) -> None:
        parser = subparsers.add_parser(
            self.get_command(), help="Synchronize assistants command"
        )

        parser.add_argument(
            "-c", "--config", type=str, help="Path to config.json file", required=True
        )
        parser.add_argument(
            "-f", "--force", action="store_true", help="Force overwrite assistant files"
        )

        # Set the default function to be called
        parser.set_defaults(func=self.execute)

    def execute(self, args: argparse.Namespace) -> None:
        self.sync_config = self._load_config(args.config)
        try:
            original_working_dir = os.getcwd()
            os.chdir(Path(args.config).parent)
            self.assistant_ids = self._load_assistant_ids()

            scout_api = ScoutAPI()

            for assistant in self.sync_config.assistants:
                self._sync_assistant(assistant, args.force, scout_api)

            self._save_deployed_ids()

        finally:
            os.chdir(original_working_dir)

    def _load_config(self, config_path: str) -> SyncConfig:
        self.config_file = Path(config_path)
        if not self.config_file.exists():
            raise FileSystemError(f"Config file not found: {config_path}")

        with open(self.config_file, "r") as f:
            try:
                config_data = json.load(f)
            except Exception as e:
                raise ModuleError(f"Error reading config file: {str(e)}")
        # Start with assistants from the config, or an empty list
        assistants = config_data.get("assistants", [])

        # Handle assistants_ref if present
        assistants_ref = config_data.pop("assistants_ref", [])
        config_dir = Path(config_path).parent
        for ref_path in assistants_ref:
            # Make ref_path relative to the config file directory
            assistant_json_ref_file_path = config_dir / Path(ref_path)
            if not assistant_json_ref_file_path.exists():
                logger.warning(
                    f"Assistant ref file not found: {assistant_json_ref_file_path}"
                )
                continue
            try:
                with open(assistant_json_ref_file_path, "r") as af:
                    assistant_data = json.load(af)
                    assistants_refs = assistant_data.get("assistants", [])
                    for a_ref in assistants_refs:
                        self._fix_assistant_paths_with_ref_path(
                            a_ref, Path(ref_path).parent
                        )
                        assistants.append(a_ref)
            except Exception as e:
                raise ModuleError(
                    f"Error loading assistant ref {assistant_json_ref_file_path}: {str(e)}"
                )

        if len(assistants) == 0:
            raise ModuleError("No assistants found in config file")

        # Replace the assistants in config_data with the merged list
        config_data["assistants"] = assistants

        try:
            return SyncConfig.model_validate(config_data, strict=True)
        except Exception as e:
            raise ModuleError(f"Error validating config data: {str(e)}")

    def _fix_assistant_paths_with_ref_path(
        self, assistant: dict, ref_path: Path
    ) -> None:
        def join_path(p: str) -> str:
            return str(ref_path / p)

        # Single path fields
        for key in ["instructions_path", "avatar_path", "secrets_path"]:
            if key in assistant and assistant[key]:
                assistant[key] = join_path(assistant[key])

        # List of paths (prompt_starters_path)
        if "prompt_starters_path" in assistant and assistant["prompt_starters_path"]:
            assistant["prompt_starters_path"] = [
                join_path(p) for p in assistant["prompt_starters_path"]
            ]

        # assistant_files: list of dicts or strings
        if "assistant_files" in assistant and assistant["assistant_files"]:
            new_files = []
            for f in assistant["assistant_files"]:
                if isinstance(f, dict) and "filepath" in f:
                    f["filepath"] = join_path(f["filepath"])
                    new_files.append(f)
                elif isinstance(f, ConfigAssistantFile) and f.filepath:
                    f.filepath = join_path(f.filepath)
                    new_files.append(f.model_dump())
            assistant["assistant_files"] = new_files

        # package_info
        if "package_info" in assistant and assistant["package_info"]:
            pkg = assistant["package_info"]
            if "package_path" in pkg and pkg["package_path"]:
                pkg["package_path"] = join_path(pkg["package_path"])

        # ui_info
        if "ui_info" in assistant and assistant["ui_info"]:
            ui = assistant["ui_info"]
            if "ui_path" in ui and ui["ui_path"]:
                ui["ui_path"] = join_path(ui["ui_path"])

    def _save_config(self, sync_config: SyncConfig) -> None:
        with open(self.config_file, "w") as f:
            json.dump(sync_config.model_dump(), f, indent=2)

    def _sync_assistant(
        self, assistant: ConfigAssistant, force: bool, scout_api: ScoutAPI
    ) -> None:
        # Check if this assistant exists
        assistant_exists, deployed_id = self._assistant_exists(assistant, scout_api)

        instructions = self._load_instructions(assistant)
        prompt_starters = self._load_prompt_starters(assistant)
        allowed_functions = self._load_allowed_functions(assistant)
        secrets = self._load_secrets(assistant)
        package_file_name = self._load_package(assistant)
        ui_package_file_name = self._load_ui_package(assistant)

        if not assistant_exists and not deployed_id:
            # Create new assistant
            logger.debug(f"Creating assistant: {assistant.name}")
            response = scout_api.create_assistant(
                name=assistant.name,
                description=assistant.description,
                instructions=instructions,
                use_system_prompt=assistant.use_system_prompt,
                prompt_starters=prompt_starters,
                allowed_functions=allowed_functions,
                variables=assistant.variables,
                secrets=secrets,
                allowed_external_services=assistant.allowed_external_services,
                visibility_type=assistant.visibility_type
                if assistant.visibility_type
                else "private",
                ui_url=assistant.ui_info
                if isinstance(assistant.ui_info, str)
                else None,
            )

            # Store the new assistant ID
            deployed_id = response.get("id")
            if not deployed_id:
                raise ModuleError("Failed to get ID for newly created assistant")

            self._store_deployed_assistant_id(assistant, deployed_id)

        avatar_url = None
        if assistant.avatar_path:
            logger.debug(
                f"Uploading avatar for assistant: {assistant.name}, id: {deployed_id}"
            )
            upload_avatar_response = scout_api.upload_assistant_avatar(
                deployed_id, assistant.avatar_path
            )
            avatar_url = upload_avatar_response.protected_url

        # update the assistant if it exists or if the avatar url is not None this will update the avatar url for newly created assistants
        if assistant_exists or avatar_url is not None:
            # Update existing assistant
            logger.debug(f"Updating assistant: {assistant.name}, id: {deployed_id}")

            scout_api.update_assistant(
                assistant_id=deployed_id,
                name=assistant.name,
                description=assistant.description,
                instructions=instructions,
                use_system_prompt=assistant.use_system_prompt,
                prompt_starters=prompt_starters,
                allowed_functions=allowed_functions,
                avatar_url=avatar_url,
                variables=assistant.variables,
                secrets=secrets,
                ui_url=assistant.ui_url,
                allowed_external_services=assistant.allowed_external_services,
                visibility_type=assistant.visibility_type,
            )

        # Sync assistant files
        self._sync_assistant_files(
            assistant, deployed_id, package_file_name, force, scout_api
        )
        self._sync_assistant_ui(assistant, deployed_id, ui_package_file_name, scout_api)

    def _load_instructions(self, assistant: ConfigAssistant) -> str:
        if assistant.instructions_text:
            return assistant.instructions_text
        elif assistant.instructions_path:
            try:
                with open(assistant.instructions_path, "r") as f:
                    return f.read()
            except FileNotFoundError:
                raise FileSystemError(
                    f"Instructions file not found: {assistant.instructions_path}"
                )
            except Exception as e:
                raise FileSystemError(f"Error reading instructions file: {str(e)}")
        return ""

    def _load_secrets(self, assistant: ConfigAssistant) -> Optional[dict]:
        if assistant.secrets:
            return assistant.secrets
        elif assistant.secrets_path:
            try:
                with open(assistant.secrets_path, "r") as f:
                    secrets = json.load(f)
                    if not isinstance(secrets, dict):
                        raise ValueError("Secrets files must be a json dictionary")
                    return secrets
            except FileNotFoundError:
                raise FileSystemError(
                    f"Secrets file not found: {assistant.secrets_path}"
                )
            except json.JSONDecodeError:
                raise ValueError(
                    f"Secrets files {assistant.secrets_path} must be a json dictionary"
                )
            except Exception as e:
                raise FileSystemError(f"Error reading secrets file: {str(e)}")
        return None

    def _load_prompt_starters(self, assistant: ConfigAssistant) -> List[str]:
        prompt_starters = []
        if assistant.prompt_starters_text:
            prompt_starters.extend(assistant.prompt_starters_text)
        if assistant.prompt_starters_path:
            for path in assistant.prompt_starters_path:
                if Path(path).exists():
                    with open(path, "r") as f:
                        prompt_starters.append(f.read())
                else:
                    logger.warning(f"Prompt starter file not found: {path}")

        return prompt_starters

    def _load_allowed_functions(self, assistant: ConfigAssistant) -> List[str] | None:
        if assistant.allowed_functions is not None:
            if len(assistant.allowed_functions) == 0:
                return []
            for function in assistant.allowed_functions:
                if function.lower() == "all":
                    return None

            return assistant.allowed_functions

        return None

    def _load_package(self, assistant: ConfigAssistant) -> Optional[str]:
        if not assistant.package_info:
            return None

        try:
            self._pkg_module._generate_package(
                assistant.package_info.package_path,
                assistant.package_info.package_file_name,
            )
        except Exception as e:
            raise ModuleError(
                f"Error generating package while synchronizing assistants package_info {assistant.package_info} : {str(e)}"
            )

        return assistant.package_info.package_file_name

    def _load_ui_package(self, assistant: ConfigAssistant) -> Optional[str]:
        if assistant.ui_url or not assistant.ui_info:
            return None

        try:
            # Run build command if specified
            if assistant.ui_info.ui_build_cmd:
                logger.debug(
                    f"Running UI build command: {assistant.ui_info.ui_build_cmd}"
                )

                subprocess.run(assistant.ui_info.ui_build_cmd, shell=True, check=True)

            ui_path = Path(assistant.ui_info.ui_path)
            if not ui_path.exists():
                raise FileSystemError(f"UI path not found: {ui_path}")

            output_zip = Path(assistant.ui_info.ui_package_file_name)
            if output_zip.exists():
                output_zip.unlink()  # Remove existing zip if present
            # Ensure ui_path is a directory
            if not ui_path.is_dir():
                raise FileSystemError(f"UI path must be a directory: {ui_path}")

            # Create zip archive from directory
            shutil.make_archive(
                str(
                    output_zip.with_suffix("")
                ),  # Remove .zip extension for make_archive
                "zip",
                ui_path,
            )

            logger.debug(
                f"UI package file name: {assistant.ui_info.ui_package_file_name}"
            )
            return assistant.ui_info.ui_package_file_name

        except subprocess.CalledProcessError as e:
            raise ModuleError(f"Error running UI build command: {str(e)}")
        except Exception as e:
            raise ModuleError(f"Error generating UI package: {str(e)}")

    def _assistant_exists(
        self, assistant: ConfigAssistant, scout_api: ScoutAPI
    ) -> tuple[bool, str]:
        # Get all existing assistants
        existing_assistants = scout_api.get_assistants()

        # Check if this assistant exists by ID (if ID is provided)
        assistant_id = self._find_assistant_id(assistant)
        if assistant_id:
            for existing in existing_assistants:
                if existing.get("id") == assistant_id:
                    return True, assistant_id
        else:
            # If no ID is provided, try matching by name
            for existing in existing_assistants:
                if existing.get("name") == assistant.name:
                    if self._api_url:
                        # Find or create the URL entry
                        url_entry = None
                        for url_ids in self._deployed_ids.ids_by_url:
                            if url_ids.url == self._api_url:
                                url_entry = url_ids
                                break

                        if not url_entry:
                            url_entry = DeployedUrlIds(url=self._api_url, ids=[])
                            self._deployed_ids.ids_by_url.append(url_entry)

                        url_entry.ids.append(
                            AssistantIds(
                                id=assistant.id, assistant_id=existing.get("id")
                            )
                        )
                    return True, existing.get("id")

        return False, ""

    def _sync_assistant_files(
        self,
        assistant: ConfigAssistant,
        deployed_id: str,
        package_file_name: Optional[str],
        force: bool,
        scout_api: ScoutAPI,
    ) -> None:
        if not assistant.assistant_files and not package_file_name:
            return

        self._upload_assistant_files(
            assistant, deployed_id, package_file_name, force, scout_api
        )
        self._edit_assistant_files(assistant, deployed_id, scout_api)

    def _upload_assistant_files(
        self,
        assistant: ConfigAssistant,
        deployed_id: str,
        package_file_name: Optional[str],
        force_overwrite: bool,
        scout_api: ScoutAPI,
    ) -> None:
        # Get existing files
        existing_files = scout_api.list_assistant_files(deployed_id)
        existing_filenames = {file.filename: file for file in existing_files}

        # Handle regular assistant files
        for file_item in assistant.assistant_files:
            if isinstance(file_item, str):
                filepath = file_item
            else:
                filepath = file_item.filepath

            file_path = Path(filepath)
            if not file_path.exists():
                logger.warning(f"Assistant file not found: {filepath}")
                continue

            filename = file_path.name
            existing_file = existing_filenames.get(filename)

            if existing_file:
                if force_overwrite:
                    self._overwrite_assistant_file(
                        deployed_id, file_path, existing_file.id, scout_api
                    )
            else:
                self._upload_new_assistant_file(deployed_id, file_path, scout_api)

        # Handle package file (always overwrite if present)
        if package_file_name:
            package_path = Path(package_file_name)
            if not package_path.exists():
                logger.warning(f"Package file not found: {package_file_name}")
            else:
                existing_package_file = existing_filenames.get(package_path.name)
                if existing_package_file:
                    self._overwrite_assistant_file(
                        deployed_id, package_path, existing_package_file.id, scout_api
                    )
                else:
                    self._upload_new_assistant_file(
                        deployed_id, package_path, scout_api
                    )

    def _overwrite_assistant_file(
        self, assistant_id: str, file_path: Path, file_uid: str, scout_api: ScoutAPI
    ) -> None:
        logger.debug(f"Force replacing file: {file_path.name}")
        scout_api.delete_assistant_file(assistant_id=assistant_id, file_uid=file_uid)
        scout_api.upload_assistant_file(
            assistant_id=assistant_id, file_path=str(file_path)
        )

    def _upload_new_assistant_file(
        self, assistant_id: str, file_path: Path, scout_api: ScoutAPI
    ) -> None:
        logger.debug(f"Uploading file: {file_path.name}")
        scout_api.upload_assistant_file(
            assistant_id=assistant_id, file_path=str(file_path)
        )

    def _edit_assistant_files(
        self, assistant: ConfigAssistant, deployed_id: str, scout_api: ScoutAPI
    ) -> None:
        existing_files = scout_api.list_assistant_files(deployed_id)
        existing_filenames = {file.filename for file in existing_files}

        # Process each file in the config
        for file_item in assistant.assistant_files:
            # if the file name match existing files, update the file

            filename = Path(file_item.filepath).name
            if filename in existing_filenames:
                for file in existing_files:
                    if file.filename == filename:
                        scout_api.edit_assistant_file(
                            assistant_id=deployed_id,
                            file_uid=file.id,
                            filename=filename,
                            description=file_item.description,
                        )

    def _sync_assistant_ui(
        self,
        assistant: ConfigAssistant,
        deployed_id: str,
        ui_package_file_name: Optional[str],
        scout_api: ScoutAPI,
    ) -> None:
        if not ui_package_file_name:
            return

        logger.debug(f"Deploying UI for assistant: {deployed_id}")
        with open(ui_package_file_name, "rb") as f:
            response = scout_api.post(
                url=f"/api/assistants/{deployed_id}/ui/deploy",
                data=EmptyRequest(),
                response_model=GenericResponse,
                files={"ui": f},
            )

        if response.status_code < 200 or response.status_code >= 300:
            raise ModuleError(f"Error deploying UI: {response.status_code}")

    def _load_assistant_ids(self) -> DeployedIds:
        # Check if we're in a dev environment based on SCOUT_CONTEXT
        is_dev_environment = (
            any(dev_url in self._api_url for dev_url in DEV_IDS_URLS)
            if self._api_url
            else False
        )
        logger.debug(
            f"Detected environment from SCOUT_CONTEXT: {'dev' if is_dev_environment else 'prod'}"
        )

        # Check for dev file first
        file = (
            Path(DEV_ASSISTANT_IDS_FILENAME)
            if is_dev_environment
            else Path(ASSISTANT_IDS_FILENAME)
        )
        self._deployed_ids_file = file
        if file.exists():
            with open(file, "r") as f:
                try:
                    return DeployedIds.model_validate(json.load(f))
                except Exception as e:
                    raise ModuleError(f"Error reading {file}: {str(e)}")
        else:
            logger.warning("No assistant_ids file found.")
            return DeployedIds(ids_by_url=[])

    def _find_assistant_id(self, assistant: ConfigAssistant) -> str | None:
        if not self._api_url:
            return None

        # Find the matching URL entry
        for url_ids in self._deployed_ids.ids_by_url:
            if url_ids.url == self._api_url:
                # Find the matching assistant ID
                for assistant_id in url_ids.ids:
                    if assistant_id.id == assistant.id:
                        return assistant_id.assistant_id
        return None

    def _save_deployed_ids(self) -> None:
        with open(self._deployed_ids_file, "w") as f:
            json.dump(self._deployed_ids.model_dump(), f, indent=2)

    def _store_deployed_assistant_id(
        self, assistant: ConfigAssistant, deployed_id: str
    ) -> None:
        if self._api_url:
            # Find or create the URL entry
            url_entry = None
            for url_ids in self._deployed_ids.ids_by_url:
                if url_ids.url == self._api_url:
                    url_entry = url_ids
                    break

            if not url_entry:
                url_entry = DeployedUrlIds(url=self._api_url, ids=[])
                self._deployed_ids.ids_by_url.append(url_entry)

            url_entry.ids.append(
                AssistantIds(id=assistant.id, assistant_id=deployed_id)
            )



"""API module for the Scout SDK."""

from .api import ScoutAPI, Response, AssistantData, AssistantDataList
from .types.assistants import AssistantResponse
from .types.chat import (
    ChatCompletionMessage,
    ChatCompletionMessageTextContent,
    ChatCompletionMessageImageContent,
    ChatCompletionMessagePDFContent,
)
from .types.images import (
    ImageRequest,
    ImageResponse,
    ImageQuality,
    ImageAspectRatio,
    ImageBackground,
)
from .constants import SCOUT_CUSTOM_FUNCTION_CONTENT_TYPE
from .utils import upload_file_to_signed_url


__all__ = [
    "ScoutAPI",
    "AssistantResponse",
    "SCOUT_CUSTOM_FUNCTION_CONTENT_TYPE",
    "Response",
    "upload_file_to_signed_url",
    "AssistantData",
    "AssistantDataList",
    # chat
    "ChatCompletionMessage",
    "ChatCompletionMessageTextContent",
    "ChatCompletionMessageImageContent",
    "ChatCompletionMessagePDFContent",
    # images
    "ImageRequest",
    "ImageResponse",
    "ImageQuality",
    "ImageAspectRatio",
    "ImageBackground",
]



"""Scout SDK - Python client library for Scout."""

from .api import ScoutAPI
from .shared.assistants_types import AssistantFile
from .shared.conversations_types import SignedUploadUrlResponse
from .shared.defines import VariableNames
from .api.types.assistants import AssistantResponse
from .api.constants import SCOUT_CUSTOM_FUNCTION_CONTENT_TYPE
from .api.project_helpers import scout
from .shared.document_chunker_types import (
    ChunkMetadata,
    Chunk,
    DocumentChunks,
    AbstractDocumentChunker,
)
from .shared.audio_types import AudioTranscriptionResponse
from .shared.protected_types import SignedUrlResponse

# Exposing CLI components
from .cli import ScoutCLI

__all__ = [
    # API components
    "ScoutAPI",
    "AssistantResponse",
    "SCOUT_CUSTOM_FUNCTION_CONTENT_TYPE",
    # CLI components
    "ScoutCLI",
    # shared components
    "AssistantFile",
    "SignedUploadUrlResponse",
    "VariableNames",
    "scout",
    # Types
    "ChunkMetadata",
    "Chunk",
    "DocumentChunks",
    "AbstractDocumentChunker",
    "AudioTranscriptionResponse",
    "SignedUrlResponse",
]



import os
from typing import Any
from dotenv import load_dotenv


CONFIG_API_ACCESS_TOKEN = "API_ACCESS_TOKEN "
CONFIG_API_URL = "API_URL"
CONFIG_ASSISTANT_ID = "ASSISTANT_ID"
CONFIG_KEYS = {CONFIG_API_ACCESS_TOKEN, CONFIG_API_URL, CONFIG_ASSISTANT_ID}


class Config:
    def __init__(self) -> None:
        """Initialize the config for the scoutcli."""
        # Load environment variables from .env file if it exists
        load_dotenv()
        self.config = {key: os.getenv(f"{key}") for key in CONFIG_KEYS}

    def get(self, key: str, default: Any = None) -> Any:
        """Get a value from the config."""
        return self.config.get(key, default)



import sys
import logging
import traceback
from typing import TypeVar, Callable, Any

# Configure logging
logging.basicConfig(level=logging.INFO, format="%(levelname)s: %(message)s")
logger = logging.getLogger("scout_cli")

# Type variable for functions
T = TypeVar("T")


class ScoutCLIError(Exception):
    """Base exception class for Scout CLI errors."""

    pass


class ConfigurationError(ScoutCLIError):
    """Exception raised for configuration errors."""

    pass


class APIError(ScoutCLIError):
    """Exception raised for API-related errors."""

    pass


class ValidationError(ScoutCLIError):
    """Exception raised for validation errors."""

    pass


class FileSystemError(ScoutCLIError):
    """Exception raised for filesystem-related errors."""

    pass


class ModuleError(ScoutCLIError):
    """Exception raised for module-related errors."""

    pass


def handle_exception(func: Callable[..., T]) -> Callable[..., T]:
    """
    Decorator to handle exceptions in CLI functions.
    Catches exceptions, logs them, and exits with an error code.

    Args:
        func: The function to wrap

    Returns:
        The wrapped function
    """

    def wrapper(*args: Any, **kwargs: Any) -> T:
        try:
            return func(*args, **kwargs)
        except KeyboardInterrupt:
            logger.info("\nOperation cancelled by user")
            sys.exit(130)  # Standard exit code for SIGINT
        except ScoutCLIError as e:
            logger.error(f"{type(e).__name__}: {str(e)}")
            if logger.level <= logging.DEBUG:
                logger.debug(
                    f"Traceback:\n{''.join(traceback.format_tb(e.__traceback__))}"
                )
            sys.exit(1)
        except Exception as e:
            logger.error(f"Unexpected error: {type(e).__name__}: {str(e)}")
            if logger.level <= logging.DEBUG:
                logger.exception("Detailed traceback:")
            sys.exit(1)

    return wrapper


def exit_with_error(message: str, exit_code: int = 1) -> None:
    """
    Log an error message and exit with the specified code.
    Deprecated: Use raise ValidationError(message) instead.

    Args:
        message: The error message to log
        exit_code: The exit code to use (default 1)
    """
    raise ValidationError(message)


def set_verbosity(verbose: bool = False) -> None:
    """
    Set the verbosity level of the logger.

    Args:
        verbose: If True, set log level to DEBUG, otherwise INFO
    """
    logger.setLevel(logging.DEBUG if verbose else logging.INFO)



import json
import mimetypes
from typing import Optional, TypeVar, Any, Type, Generic
from pydantic import BaseModel
import requests

from .request_utils import RequestUtils
from .types.assistants import AssistantResponse
from .types.chat import ChatCompletionMessage
from ..shared.assistants_types import (
    AssistantFile,
    AssistantFileUploadResponse,
    AssistantUploadImageResponse,
)
from ..shared.conversations_types import SignedUploadUrlResponse
from ..shared.defines import VariableNames
from ..shared.protected_types import SignedUrlResponse
from .project_helpers import scout

DEFAULT_MODEL = "gpt-4o"

# Create generic type variables for request and response
RequestType = TypeVar("RequestType", bound=BaseModel)
ResponseType = TypeVar("ResponseType", bound=BaseModel)
ResponseFormatType = TypeVar("ResponseFormatType", bound=BaseModel)


class Response(Generic[ResponseType]):
    def __init__(self, status_code: int, data: ResponseType):
        self.status_code = status_code
        self.data = data


class AssistantData(BaseModel):
    metadata: dict
    content: str
    embedding: Optional[list] = None


class AssistantDataList(BaseModel):
    list: list[AssistantData]


class ScoutAPI:
    def __init__(
        self,
        base_url: Optional[str] = None,
        api_access_token: Optional[str] = None,
    ) -> None:
        self._base_url = base_url or scout.context.get(VariableNames.SCOUT_API_URL)
        if self._base_url is None or self._base_url == "":
            raise ValueError(
                f"{VariableNames.SCOUT_API_URL} is not set in SCOUT_CONTEXT"
            )

        api_access_token = api_access_token or scout.context.get(
            VariableNames.SCOUT_API_ACCESS_TOKEN
        )
        if api_access_token is None or api_access_token == "":
            raise ValueError(
                f"{VariableNames.SCOUT_API_ACCESS_TOKEN} is not set in SCOUT_CONTEXT"
            )

        self._headers = {
            "Authorization": f"Bearer {api_access_token}",
            "Content-Type": "application/json",
        }

    def _get_validated_data(
        self, response: Any, response_model: Optional[Type[ResponseType]] = None
    ) -> Any:
        if response_model and isinstance(response, dict):
            validated_data = response_model.model_validate(response)
        else:
            validated_data = response

        return validated_data

    # Generic requests
    def get(
        self,
        url: str,
        params: RequestType,
        response_model: Optional[Type[ResponseType]] = None,
    ) -> Response[Any]:
        request_data = params.model_dump() if isinstance(params, BaseModel) else params

        response, status_code = RequestUtils.get(
            url=f"{self._base_url}{url}",
            headers=self._headers,
            params=request_data,
        )

        validated_data = self._get_validated_data(response, response_model)

        return Response(status_code=status_code, data=validated_data)

    def put(
        self,
        url: str,
        data: RequestType,
        response_model: Optional[Type[ResponseType]] = None,
    ) -> Response[Any]:
        request_data = data.model_dump() if isinstance(data, BaseModel) else data

        response, status_code = RequestUtils.put(
            url=f"{self._base_url}{url}", headers=self._headers, payload=request_data
        )

        validated_data = self._get_validated_data(response, response_model)

        return Response(status_code=status_code, data=validated_data)

    def post(
        self,
        url: str,
        data: RequestType | dict,
        response_model: Optional[Type[ResponseType]] = None,
        files: Optional[dict] = None,
    ) -> Response[Any]:
        request_data = data.model_dump() if isinstance(data, BaseModel) else data

        if files:
            local_headers = self._headers.copy()
            local_headers.pop("Content-Type")

            response, status_code = RequestUtils.post(
                url=f"{self._base_url}{url}",
                headers=local_headers,
                data=request_data,
                files=files,
            )
        else:
            response, status_code = RequestUtils.post(
                url=f"{self._base_url}{url}",
                headers=self._headers,
                json_payload=request_data,
            )

        validated_data = self._get_validated_data(response, response_model)

        return Response(status_code=status_code, data=validated_data)

    def delete(
        self, url: str, response_model: Optional[Type[ResponseType]] = None
    ) -> Response[Any]:
        response, status_code = RequestUtils.delete(
            url=f"{self._base_url}{url}",
            headers=self._headers,
        )

        validated_data = self._get_validated_data(response, response_model)

        return Response(status_code=status_code, data=validated_data)

    # Embeddings

    def create_embeddings(self, texts: list[str]) -> list[list[float]]:
        response, status_code = RequestUtils.post(
            url=f"{self._base_url}/api/embeddings/",
            headers=self._headers,
            json_payload={"texts": texts},
        )

        return response

    # TODO create a method that takes an array of message
    def chat_completion(
        self,
        messages: list[ChatCompletionMessage] | str,
        model: Optional[str] = None,
        assistant_id: Optional[str] = None,
        stream: Optional[bool] = None,
        response_format: Optional[Type[ResponseFormatType]] = None,
        debug: Optional[bool] = False,
        allowed_tools: Optional[list[str]] = None,
        llm_args: Optional[dict] = None,
    ) -> Any:
        model = model or DEFAULT_MODEL
        stream = stream or False

        if isinstance(messages, str):
            messages = [ChatCompletionMessage(role="user", content=messages)]

        payload = {
            "messages": [message.model_dump() for message in messages],
            "model": model,
            "stream": stream,
            **({"assistant_id": assistant_id} if assistant_id else {}),
            **({"allowed_tools": allowed_tools} if allowed_tools is not None else {}),
            **({"llm_args": llm_args} if llm_args else {}),
        }

        if response_format:
            payload["response_format"] = response_format.model_json_schema()

        if debug:
            print(f"payload: {payload}")

        response, status_code = RequestUtils.post(
            url=f"{self._base_url}/api/chat/completion/",
            headers=self._headers,
            json_payload=payload,
            stream=stream,
        )

        if response_format:
            # extract the last message from the response to ignore tools calls
            try:
                if stream:
                    content = response.get("content", "")
                else:
                    content = response.get("messages", [])[-1].get("content", "")

                return response_format.model_validate(json.loads(content))
            except Exception as e:
                raise Exception(f"Error processing Response: {response}") from e

        return response

    # Assistants

    def create_assistant(
        self,
        name: str,
        description: str,
        instructions: str,
        use_system_prompt: bool = True,
        prompt_starters: list[str] = [],
        visibility_type: str = "private",
        avatar_url: Optional[str] = None,
        allowed_functions: Optional[list[str]] = None,
        variables: Optional[dict[str, str]] = None,
        secrets: Optional[dict[str, str]] = None,
        allowed_external_services: Optional[list[str]] = None,
        ui_url: Optional[str] = None,
    ) -> Any:
        payload = {
            "name": name,
            "description": description,
            "instructions": instructions,
            "use_system_prompt": use_system_prompt,
            "prompt_starters": prompt_starters,
            "visibility": {"type": visibility_type},
            "avatar_url": avatar_url,
            **({"variables": variables} if variables is not None else {}),
            **({"secrets": secrets} if secrets is not None else {}),
            **(
                {"allowed_functions": allowed_functions}
                if allowed_functions is not None
                else {}
            ),
            **(
                {"allowed_external_services": allowed_external_services}
                if allowed_external_services is not None
                else {}
            ),
            **({"ui_url": ui_url} if ui_url is not None else {}),
        }

        response, status_code = RequestUtils.post(
            url=f"{self._base_url}/api/assistants/",
            headers=self._headers,
            json_payload=payload,
        )

        return response

    def update_assistant(
        self,
        assistant_id: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        instructions: Optional[str] = None,
        use_system_prompt: Optional[bool] = True,
        prompt_starters: Optional[list[str]] = None,
        visibility_type: Optional[str] = "private",
        avatar_url: Optional[str] = None,
        ui_url: Optional[str] = None,
        links: Optional[list[str]] = None,
        allowed_functions: Optional[list[str]] = None,
        variables: Optional[dict[str, str]] = None,
        secrets: Optional[dict[str, Optional[str]]] = None,
        allowed_external_services: Optional[list[str]] = None,
    ) -> Any:
        payload = {
            **({"name": name} if name is not None else {}),
            **({"description": description} if description is not None else {}),
            **({"instructions": instructions} if instructions is not None else {}),
            **(
                {"use_system_prompt": use_system_prompt}
                if use_system_prompt is not None
                else {}
            ),
            **(
                {"prompt_starters": prompt_starters}
                if prompt_starters is not None
                else {}
            ),
            **(
                {"visibility": {"type": visibility_type}}
                if visibility_type is not None
                else {}
            ),
            **({"avatar_url": avatar_url} if avatar_url is not None else {}),
            **({"ui_url": ui_url} if ui_url is not None else {}),
            **({"links": links} if links is not None else {}),
            **({"variables": variables} if variables is not None else {}),
            **({"secrets": secrets} if secrets is not None else {}),
            **(
                {"allowed_functions": allowed_functions}
                if allowed_functions is not None
                else {}
            ),
            **(
                {"allowed_external_services": allowed_external_services}
                if allowed_external_services is not None
                else {}
            ),
        }

        response, status_code = RequestUtils.patch(
            url=f"{self._base_url}/api/assistants/{assistant_id}",
            headers=self._headers,
            json_payload=payload,
        )

        return response

    def get_assistants(self) -> Any:
        response, status_code = RequestUtils.get(
            url=f"{self._base_url}/api/assistants", headers=self._headers
        )

        return response

    def upload_assistant_avatar(
        self,
        assistant_id: str,
        file_path: str,
    ) -> AssistantUploadImageResponse:
        with open(file_path, "rb") as f:
            content_type = mimetypes.guess_type(file_path)[0]
            files = {"file": (file_path, f, content_type)}
            local_headers = self._headers.copy()
            local_headers.pop("Content-Type")

            response, status_code = RequestUtils.post(
                url=f"{self._base_url}/api/assistants/{assistant_id}/avatar/upload",
                headers=local_headers,
                files=files,
            )
        return AssistantUploadImageResponse(**response)

    def upload_assistant_file(
        self,
        assistant_id: str,
        file_path: str,
    ) -> AssistantFileUploadResponse:
        with open(file_path, "rb") as f:
            files = {"file": f}
            local_headers = self._headers.copy()
            local_headers.pop("Content-Type")

            response, status_code = RequestUtils.post(
                url=f"{self._base_url}/api/assistants/{assistant_id}/files",
                headers=local_headers,
                files=files,
            )
        return AssistantFileUploadResponse(**response)

    def list_assistant_files(
        self,
        assistant_id: str,
    ) -> list[AssistantFile]:
        response, status_code = RequestUtils.get(
            url=f"{self._base_url}/api/assistants/{assistant_id}/files",
            headers=self._headers,
        )
        return [AssistantFile(**file) for file in response]

    def edit_assistant_file(
        self,
        assistant_id: str,
        file_uid: str,
        filename: str = "Default",
        description: Optional[str] = None,
    ) -> Any:
        data = {}
        data.update({"file_name": filename})
        data.update({"file_description": description}) if description else None

        response, status_code = RequestUtils.put(
            url=f"{self._base_url}/api/assistants/{assistant_id}/files/{file_uid}",
            headers=self._headers,
            payload=data,
        )
        return response

    def delete_assistant_file(
        self,
        assistant_id: str,
        file_uid: str,
    ) -> AssistantResponse:
        response, status_code = RequestUtils.delete(
            url=f"{self._base_url}/api/assistants/{assistant_id}/files/{file_uid}",
            headers=self._headers,
        )
        return AssistantResponse(**response)

    # Conversations

    def create_conversation(
        self,
        assistant_id: Optional[str] = None,
        model: Optional[str] = None,
        title: Optional[str] = None,
        payload: Optional[dict[str, Any]] = None,
        time_zone_offset: str = "-0400",
    ) -> Any:
        request_payload: dict[str, Any] = {}
        request_payload.update({"assistant_id": assistant_id}) if assistant_id else None
        request_payload.update({"model": model if model else DEFAULT_MODEL})
        request_payload.update({"title": title}) if title else None
        request_payload.update({"payload": payload}) if payload else None
        request_payload.update({"time_zone_offset": time_zone_offset})

        response, status_code = RequestUtils.post(
            url=f"{self._base_url}/api/conversations/",
            headers=self._headers,
            json_payload=request_payload,
        )
        return response

    def search_assistant_data(
        self,
        assistant_id: str,
        query: str,
        strategy: Optional[dict] = None,
        where: Optional[dict] = None,
    ) -> Any:
        response, status_code = RequestUtils.post(
            url=f"{self._base_url}/api/assistants/{assistant_id}/search",
            headers=self._headers,
            json_payload={"query": query, "strategy": strategy, "where": where},
        )
        return response

    def create_assistant_data(
        self, assistant_id: str, data: AssistantData | AssistantDataList
    ) -> Any:
        data_list = (
            data.model_dump().get("list")
            if isinstance(data, AssistantDataList)
            else [data.model_dump()]
        )

        response, status_code = RequestUtils.post(
            url=f"{self._base_url}/api/assistants/{assistant_id}/data",
            headers=self._headers,
            json_payload={"data": data_list},
        )
        return response

    def update_assistant_data(
        self,
        assistant_id: str,
        data_id: str,
        metadata: dict,
        content: str,
        embedding: Optional[list] = None,
    ) -> Any:
        response, status_code = RequestUtils.put(
            url=f"{self._base_url}/api/assistants/{assistant_id}/data/{data_id}",
            headers=self._headers,
            payload={"metadata": metadata, "content": content, "embedding": embedding},
        )
        return response

    def query_assistant_data(self, assistant_id: str, where: dict) -> Any:
        response, status_code = RequestUtils.get(
            url=f"{self._base_url}/api/assistants/{assistant_id}/data",
            headers=self._headers,
            params=where,
        )
        return response

    def delete_assistant_data(
        self, assistant_id: str, id: Optional[str] = None, where: Optional[dict] = None
    ) -> Any:
        if id is None and where is None:
            raise ValueError("Either 'id' or 'where' must be provided.")

        response, status_code = RequestUtils.delete(
            url=f"{self._base_url}/api/assistants/{assistant_id}/data",
            headers=self._headers,
            json_payload={"where": where, "id": id},
        )
        return response

    def get_conversation_signed_upload_url(
        self,
        conversation_id: str,
        file_path: str,
    ) -> SignedUploadUrlResponse:
        payload = {"file_path": file_path}

        response, status_code = RequestUtils.post(
            url=f"{self._base_url}/api/conversations/{conversation_id}/signed-upload-url",
            headers=self._headers,
            json_payload=payload,
        )
        try:
            return SignedUploadUrlResponse.model_validate(response)
        except Exception as e:
            raise ValueError(f"Invalid response: {response}") from e

    def get_signed_url(
        self,
        path: str,
    ) -> SignedUrlResponse:
        response, status_code = RequestUtils.get(
            url=f"{self._base_url}/api/protected/{path}",
            headers=self._headers,
        )

        return SignedUrlResponse.model_validate(response)

    def get_conversation_signed_url(
        self,
        conversation_id: str,
        file_path: str,
    ) -> SignedUrlResponse:
        response, status_code = RequestUtils.get(
            url=f"{self._base_url}/api/protected/conversations/{conversation_id}/{file_path}",
            headers=self._headers,
        )

        return SignedUrlResponse.model_validate(response)

    def chunk_document(self, file_path: str) -> dict:
        with open(file_path, "rb") as f:
            files = {"file": f}
            local_headers = self._headers.copy()
            local_headers.pop("Content-Type")

            response, status_code = RequestUtils.post(
                url=f"{self._base_url}/api/utils/chunk-document",
                headers=local_headers,
                files=files,
            )
        return response

    def get_document_text(self, file_path: str, args: Optional[dict] = None) -> dict:
        with open(file_path, "rb") as f:
            files = {"file": f}
            local_headers = self._headers.copy()
            local_headers.pop("Content-Type")

            response, status_code = RequestUtils.post(
                url=f"{self._base_url}/api/utils/get-file-text-content",
                headers=local_headers,
                files=files,
                json_payload=args,
            )
        return response

    def llm_filter_documents(
        self,
        query: str,
        context: str,
        documents: dict[str, str],
        batch_size: int = 10,
        model_id: Optional[str] = None,
    ) -> dict:
        response, status_code = RequestUtils.post(
            url=f"{self._base_url}/api/utils/llm-filter-documents",
            headers=self._headers,
            json_payload={
                "query": query,
                "context": context,
                "documents": documents,
                "batch_size": batch_size,
                "model_id": model_id,
            },
        )
        return response

    def text_to_speech(
        self,
        text: str,
        model: Optional[str] = None,
        voice: Optional[str] = None,
        api_args: Optional[dict] = None,
    ) -> bytes:
        json_payload = {
            "text": text,
            **({"model": model} if model is not None else {}),
            **({"voice": voice} if voice is not None else {}),
            **({"api_args": api_args} if api_args is not None else {}),
        }

        response = requests.post(
            url=f"{self._base_url}/api/audio/speech",
            headers=self._headers,
            json=json_payload,
        )
        return response.content

    def execute_assistant_function(
        self,
        assistant_id: str,
        function_name: str,
        payload: dict,
        response_model: Type[ResponseType],
    ) -> Response[ResponseType]:
        assistant_id = assistant_id
        api_url = f"/api/assistants/{assistant_id}/functions/{function_name}"
        result = self.post(url=api_url, data=payload, response_model=response_model)
        return result



from pydantic import BaseModel, Field
from scoutsdk.api import ScoutAPI
from scoutsdk import scout


class TypedObjectFunctionParameters(BaseModel):
    text_file_name: str = Field(
        description="Description of the field usage (Seen by the LLM)"
    )
    text_file_size: int = Field(
        description="Description of the field usage (Seen by the LLM)"
    )


@scout.function(description="Description of the function usage (Seen by the LLM)")
def this_is_a_test_function(
    my_parameter: str = Field(description="Description of the field (Seen by the LLM)"),
    my_custom_class_parameter: TypedObjectFunctionParameters = Field(
        description="Description of the field (Seen by the LLM)"
    ),
):
    scout_api = ScoutAPI()  # Instanciate a ScoutAPI. When used in an assistant, Token and URL are automatically filled.
    assistants = scout_api.get_assistants()  # Call the ScoutAPI
    print(assistants)

    variable_from_assistant = scout.context.get("variable_from_assistant")
    print(variable_from_assistant)

    secret_from_assistant = scout.context.get("secret_from_assistant")
    print(secret_from_assistant)
    return "success"



from pydantic import BaseModel, field_validator
from typing import Any
from enum import StrEnum


class UploadMode(StrEnum):
    AZURE = "azure"
    POST = "post"


class SignedUploadUrlResponse(BaseModel):
    url: str
    fields: dict[str, Any] = {}
    prefix: str = ""
    protected_url: str = ""
    upload_mode: UploadMode = UploadMode.POST
    content_type: str = ""

    @field_validator("protected_url")
    @classmethod
    def validate_protected_url(cls, protected_url: str) -> str:
        if not protected_url.startswith("/protected"):
            protected_url = f"/protected/{protected_url}"
        return protected_url



from scoutsdk.shared.conversations_types import SignedUploadUrlResponse
from scoutsdk.shared.upload_files import upload_file


def upload_file_to_signed_url(
    signed_url_response: SignedUploadUrlResponse,
    file_path: str,
    file_key: str | None = None,
) -> int:
    if file_key is None:
        file_key = file_path.split("/")[-1]

    return upload_file(signed_url_response, file_path, file_key)
